from familiar.core import *
import libglade
import gtk
import GDK
import GtkExtra
import string
import re
import time
import thread
import threading
import urllib

class Address(FamiliarClass):
    def __init__(self):
        self.mimetype = "application/x-streetaddress"
        self.physical_address = []  # street, city, state, zip, country
        self.widget = None
        self.manager = None
        
        self.stored_directions = {}
        FamiliarClass.__init__(self)

    def set_manager(self, manager):
        self.manager = manager

    def get_manager(self):
        return self.manager

    def new_view(self):
        return AddressView(self)
    
    def set_address(self, address):
        self.physical_address = address
        
    def get_address(self):
        try:
            address = "%s\n%s,%s %s" % (self.physical_address[0],self.physical_address[1],self.physical_address[2],self.physical_address[3])
        except:
            address = ""
        return address

    def set_widget(self,widget):
        self.widget = widget

    def get_widget(self):
        return self.widget
    
    def get_manager(self):
        return self.manager

    def set_manager(self, manager):
        self.manager = manager
        
    def get_directions(self,address_to):
        directions_return = []
        directions_save = []
        if self == address_to:
            directions_save.append("[Same Address]")
            return directions_save
        # Yank out the from address
        pat = re.compile("\s")

        street0=re.sub(pat,"+",self.physical_address[0])
        city0=re.sub(pat,"+",self.physical_address[1])
        state0=re.sub(pat,"+",self.physical_address[2])
        # Yank out the to address
        street1=re.sub(pat,"+",address_to.physical_address[0])
        city1=re.sub(pat,"+",address_to.physical_address[1])
        state1=re.sub(pat,"+",address_to.physical_address[2])

        urls = []
        # Fastest Route 
	urls.append("http://www.mapquest.com/cgi-bin/mqtrip?link=btwn/twn-ddir_na_basic_main&=1ADDR_0=%s&CITY_0=%s&sTATE_0=%s&CC_0=US&ADDR_1=%s&CITY_1=%s&STATE_1=%s&CC_1=US" % (street0,city0,state0,street1,city1,state1))
        # Shortest Route
	urls.append("http://www.mapquest.com/cgi-bin/mqtrip?link=btwn/twn-ddir_na_basic_main&shortest.x=1addr_0=%s&city_0=%s&state_0=%s&cc_0=US&addr_1=%s&city_1=%s&state_1=%s&cc_1=US" % (street0,city0,state0,street1,city1,state1))
        # Route avoiding freeways
	urls.append("http://www.mapquest.com/cgi-bin/mqtrip?link=btwn/twn-ddir_na_basic_main&avoid_highway.x=1addr_0=%s&city_0=%s&state_0=%s&cc_0=US&addr_1=%s&city_1=%s&state_1=%s&cc_1=US" % (street0,city0,state0,street1,city1,state1))

        threads = []
        for url in urls:
            t = FetchUrlThread(url)
            t.start()
            threads.append(t)
            
        for thread in threads:
            thread.join()
            
        for thread in threads:
            directions_return.append(thread.directions_save)
            
        return directions_return

class AddressView(FamiliarClass):
    def __init__(self,object):
        self.object = object
        self.callback_object_list = []
        FamiliarClass.__init__(self)

    # XXX - This needs to become a clean API that all of the object
    # views share.
    
    def register_callback(self, callback_object, type=None):
        self.callback_object_list.append(callback_object)

    def notify_registered_callbacks(self, type=None,object=None):
        for callback in self.callback_object_list:
            callback.notify_callback(type,object)

    def view(self):
        self.helper = GladeHelper("familiar/address.glade",
                                  AddressSignalHandler)

        addresswindow = self.helper["addresswindow"]
        addresswindow.set_data("view",self)
        
        street = self.helper["street"]
        city = self.helper["city"]
        state = self.helper["state"]
        zip = self.helper["zip"]

        okay = self.helper["okay"]
        okay.set_data("view",self)
        
        Cancel = self.helper["Cancel"]
        Cancel.set_data("view",self)
        
        address = self.object.physical_address
        
        if len(address) >=1:
            street.set_text(address[0])
        if len(address) >=2:
            city.set_text(address[1])
        if len(address) >=3:
            state.set_text(address[2])
        if len(address) >=4:
            zip.set_text(address[3])

    def save_address(self):

        street = self.helper["street"]
        city = self.helper["city"]
        state = self.helper["state"]
        zip = self.helper["zip"]

        old_address_list = self.object.physical_address
        
        new_address_list = []
        new_address_list.append(street.get_text())
        new_address_list.append(city.get_text())
        new_address_list.append(state.get_text())
        new_address_list.append(zip.get_text())

        if old_address_list == new_address_list:
            #
            # We didnt change anything so lets go back
            #
            print "No change"
            self.notify_registered_callbacks("cancel",self.object)
            self.helper["addresswindow"].destroy()
            return

        #
        # Gather the current list of address
        #
        manager = self.object.get_manager()
        addresses = manager.search_by_type("application/x-streetaddress")

        for address in addresses:
           print address.physical_address
           if address.physical_address == new_address_list:
               print "Match found"
               #
               # If we have a match we use the existing object
               # we attach the existing address object to the widget
               # It'll get removed once we get called back
               #
               address.set_widget(self.object.get_widget())
               manager.unregister_object(self.object)
               self.notify_registered_callbacks("address",address)
               self.helper["addresswindow"].destroy()
               return
               
        #
        # We are creating a new address object and the object is created
        # so we just add the address to it.
        #
        self.object.set_address(new_address_list)
        self.notify_registered_callbacks("address",self.object)
        self.helper["addresswindow"].destroy()
        return

    def cancel_address(self):
        self.notify_registered_callbacks("cancel",self.object)
        self.helper["addresswindow"].destroy()
        
            

class GladeHelper:
    def __init__(self, glade_file, handler):
        self.widgets = libglade.GladeXML(glade_file)
        self.widgets.signal_autoconnect(handler.__dict__)

    def __getitem__(self, key):
        return self.widgets.get_widget(key)
    

class AddressSignalHandler:
    def on_okay_clicked(widget,*args):
        view = widget.get_data("view")
        view.save_address()
        
    def on_Cancel_clicked(widget,*args):
        view = widget.get_data("view")
        view.cancel_address()
    def on_addresswindow_destroy(widget,*args):
        view = widget.get_data("view")
        view.cancel_address()
        
class FetchUrlThread(threading.Thread):
    def __init__(self,url):
        threading.Thread.__init__(self)
        self.url = url
        self.directions_save = []
    def run(self):

        try:
            f = urllib.urlopen(self.url)
        except:
            pass
        #    return None
        # Direction lines regexs
        pat = re.compile("\s+\d+\:$")
        pat2 = re.compile("^\s+")
        pat3 = re.compile("\&nbsp\;\s+")
        # Total lines regexs
        total_time_pat = re.compile("^\s+\&nbsp\;TOTAL ESTIMATED")
        self.contents = f.readlines()
        self.matches = []
        found_extras = 0
        extras = []
        count = 0
        for self.content in self.contents:
            m = pat.match(self.content)
            m2 = total_time_pat.match(self.content)
            if m:
                self.content = re.sub(pat2,"",self.content)
                self.content = re.sub("$\n","",self.content)
                # Grab the directions
                self.contents[count+3] = re.sub(pat2,"",self.contents[count+3])
                self.contents[count+3] = re.sub("</td>","",self.contents[count+3])
                # Grab the milage
                self.contents[count+5] = re.sub(pat2,"",self.contents[count+5])
                tmp = self.contents[count+5]
                self.contents[count+5] = re.sub(pat3,"",tmp)
                self.contents[count+5] = re.sub("\n","",self.contents[count+5])
                line = []
                line.append(self.content)
                line.append(self.contents[count+3])
                line.append(self.contents[count+5])
                line.append("\n")
                self.matches.append(line)
            elif m2:
                if found_extras == 0:
                    self.contents[count+10] = re.sub("</td>","",self.contents[count+10])
                    self.contents[count+10] = re.sub("\s+\&nbsp\;","",self.contents[count+10])
                    self.contents[count+16] = re.sub(pat3,"",self.contents[count+16])
                    total_time = "Total Est. Time: %s" % (self.contents[count+10])
                    total_distance = "Total Distance: %s" % (self.contents[count+16])
                    found_extras=1
            else:
                pass
            count = count+1
        if len(self.matches):
            for match in self.matches:
                first = "%s %s" % (match[0],match[1])
                second = "Distance: %s\n" % (match[2])
                self.directions_save.append(first)
                self.directions_save.append(second)
            #
            # Add the total time and distance
            #
            self.directions_save.append(total_time)
            self.directions_save.append(total_distance)
#        else:
#            return None
#        self.semaphore.release()
#        print self.semaphore
#        return self.directions_save      
