import libglade
import gtk
import GDK
import GtkExtra
import string
import re
import time
import thread
import math

from core import *
import urllib 
from image import *
import address

not_available_string = "[n/a]"

class Contact(FamiliarClass):
    def __init__(self):
        self.mimetype = "application/x-contact"
        self.name = [] # Format should be first,last,middle,title,company,nickname
        self.physical_addresses = []
        self.phone_numbers = []
        self.email_addresses = []
        self.birthdate = None
        self.image = None
        self.manager = None

    def set_manager(self, manager):
        self.manager = manager

    def get_manager(self):
        return self.manager
    
    def set_name(self, name):
        for i in range(0,len(name)):
            if i >= len(self.name):
                self.name.append(name[i])
            else:
                self.name[i] = name[i]

    def get_owner(self):
        contact_manager = self.manager.search_by_type("application/x-contactmanager")[0]
        return contact_manager.get_owner()
        
    def get_name(self):
        return self.name[:]

    def set_image(self, image):
        self.image = image

    def get_image(self):
        return self.image

    def set_physical_addresses(self, addresses):
        self.physical_addresses = addresses
        
    def get_physical_addresses(self):
        return self.physical_addresses

    def set_phone_numbers(self, phone_numbers):
        self.phone_numbers = phone_numbers
        
    def get_phone_numbers(self):
        return self.phone_numbers

    def set_email_addresses(self, email_addresses):
        self.email_addresses = email_addresses

    def get_email_addresses(self):
        return self.email_addresses

    def set_birthdate(self, birthdate):
        self.birthdate = birthdate
        
    def get_birthdate(self):
        return self.birthdate

    def new_view(self):
        # XXX - This should be routed through the ObjectManager
        return ContactView(self)

    def search_addresses(self,text):

        matches = []
        addresses = self.manager.search_by_type("application/x-streetaddress")
        count = 0
        for address in addresses:
            address_string = address.get_address()
            if address_string == text:
                return address
            
        return None
    
        
        
    def get_printable_name(self):
        # XXX - This needs to be moved into the view.
        name = self.get_name()
        if name[1]:
            full_name = string.join([name[1], name[0]], ", ")
        else:
            full_name = name[0]
        
        return full_name
    

class ContactView(FamiliarClassView):
    def __init__(self, object):
        self.object = object
        self.callback_object_list = []
        
        self.previous_address_strings = []
        FamiliarClassView.__init__(self)

    # XXX - This needs to become a clean API that all of the object
    # views share.
    
    def register_callback(self, callback_object, type=None):
        self.callback_object_list.append(callback_object)

    def notify_registered_callbacks(self, *args):
        for callback in self.callback_object_list:
            callback.notify_callback(self)
        
    def view(self,function):
        self.helper = GladeHelper("familiar/contact.glade",
                                  ContactSignalHandler)

        contactwindow = self.helper["contactwindow"]
        contactwindow.set_default_size(240,320)
        
        width=gtk.screen_width()
        height=gtk.screen_height()
        if width >=640 and height >= 480:
            pass
        else:
            width=width-20
            height=height-40
            contactwindow.set_usize(width,height)

        edit_button = self.helper["edit_button"]
        edit_button.set_data("view",self)

        cancel_button = self.helper["cancel_button"]
        cancel_button.set_data("view",self)

        save_button = self.helper["save_button"]
        save_button.set_data("view",self)

        close_button = self.helper["close_button"]
        close_button.set_data("view",self)

        close2_button = self.helper["close2_button"]
        close2_button.set_data("view",self)

        if function == "view":
            self.refresh_view()
        elif function == "edit":
            self.edit_contact_tab()
        else:
            pass

    def refresh_view(self):
        
        main_frame = self.helper["main_frame"]
        main_frame.set_label(self.object.get_printable_name())
        
        addresses = []
        addresses = self.object.get_physical_addresses()
        print addresses
        addresses_frame = self.helper["addresses_frame"]
        address_list = self.helper["address_list"]
        
        address_list.clear_items(0,len(address_list.children()))
        item_list = []

        tmp = 0
        for address in addresses:    
            if not address:
                continue
            
            widget = gtk.GtkListItem(address.get_address())
            widget.set_data("address",address)
            widget.set_data("view",self)
            widget.show()
            item_list.append(widget)
            tmp += 1
            
        addresses_frame.show()
    
        if not tmp:
            widget = gtk.GtkListItem(not_available_string)
            widget.show()
            item_list.append(widget)
            
        address_list.append_items(item_list)
        addresses_frame.show()

        phone_numbers = []
        phone_numbers = self.object.get_phone_numbers()
        phone_numbers_list = self.helper["phone_list"]
        phone_numbers_frame = self.helper["phone_numbers_frame"]

        phone_numbers_list.clear_items(0,len(phone_numbers_list.children()))
        item_list = []

        tmp = 0
        for phone_number in phone_numbers:
            if not phone_number:
                continue
            
            widget = gtk.GtkListItem(phone_number)
            widget.show()
            item_list.append(widget)
            tmp += 1

        if not tmp:
            widget = gtk.GtkListItem(not_available_string)
            widget.show()
            item_list.append(widget)

	phone_numbers_list.append_items(item_list)
	phone_numbers_frame.show()

	email_addresses = self.object.get_email_addresses()
	email_addresses_frame = self.helper["email_addresses_frame"]
	email_addresses_list = self.helper["email_list"]

	email_addresses_list.clear_items(0,len(email_addresses_list.children()))
	item_list = []

	tmp = 0
	for email in email_addresses:
		if not email:
			continue

		widget = gtk.GtkListItem(email)
		widget.show()
		item_list.append(widget)
		tmp += 1
	
	if not tmp:
		widget = gtk.GtkListItem(not_available_string)
		widget.show()
		item_list.append(widget)

	email_addresses_list.append_items(item_list)
	email_addresses_frame.show()

	birthdate = self.object.get_birthdate()
	birthdate_label = self.helper["birthdate_label"]
	birthdate_frame = self.helper["birthdate_frame"]

	if birthdate:
		birthdate_label.set_text(birthdate)
	else:
		birthdate_label.set_text(not_available_string)

	# XXX -- Hokey as hell.
        image = self.object.get_image()
        picture_frame = self.helper["picture_frame"]

        widget = image.new_view().get_widget()
        widget.show()
        children = []
        children = picture_frame.children()
        if len(children):
            picture_frame.remove(children[0])
        picture_frame.add(widget)

    def edit_contact_tab(self):
        name = []
        address = []
        physical_address = []
        phones_numbers = []
        email_addresses = []

        self.helper["name_button"].set_data("view",self)
        self.helper["name_button"].set_data("name","name")
        
        self.helper["address_button"].set_data("view",self)
        self.helper["address_button"].set_data("name","address")
        
        self.helper["phone_button"].set_data("view",self)
        self.helper["phone_button"].set_data("name","phone")
        
        self.helper["email_button"].set_data("view",self)
        self.helper["email_button"].set_data("name","email")
        
        self.helper["address1_1"].set_data("view",self)

        self.helper['imagebutton'].set_data("view",self)
        
        notebook = self.helper["notebook1"]
        notebook.set_page(1)

        firstname = self.helper["firstname"]
        lastname = self.helper["lastname"]
        middlename = self.helper["middlename"]
        titlename = self.helper["titlename"]
        companyname = self.helper["companyname"]
        nickname = self.helper["nickname"]

        name = self.object.get_name()
        if name:
            firstname.set_text(name[0])
            lastname.set_text(name[1])
            if len(name) >= 3:
                if name[2] != None:
                    middlename.set_text(name[2])
            if len(name) >= 4:
                if name[3] != None:
                    titlename.set_text(name[3])
            if len(name) >= 5:
                if name[4] != None:
                    companyname.set_text(name[4])
            if len(name) >= 6:
                if name[5] != None:
                    nickname.set_text(name[5])                               
        else:
            self.helper["edit_frame"].set_label("Add A New Contact")
            self.helper["cancel_button"].hide()
            self.helper["close2_button"].show()

        physical_addresses  = self.object.get_physical_addresses()

        widget_array = [ self.helper["address1_1"],
                         self.helper["address2_1"],
                         self.helper["address3_1"],
                         self.helper["address4_1"] ]

        i = 0
        for address in physical_addresses:

            if not address:
                continue

            widget_array[i].set_point(0)
            chars = widget_array[i].get_length()
            widget_array[i].forward_delete(chars)
            widget_array[i].insert_defaults(address.get_address())
            widget_array[i].set_data("address",address)

            i+=1

        phone_numbers = self.object.get_phone_numbers()
        if phone_numbers:
            phone1 = self.helper["phone1"]
            work_phone = self.helper["work_phone"]
            cell_phone = self.helper["cell_phone"]
            pager_phone = self.helper["pager_phone"]
            
            if len(phone_numbers) >= 1:
                if phone_numbers[0] != None:
                    phone1.set_text(phone_numbers[0])
            if len(phone_numbers) >= 2:
                if phone_numbers[1] != None:
                    work_phone.set_text(phone_numbers[1])
            if len(phone_numbers) >= 3:
                if phone_numbers[2] != None:
                    cell_phone.set_text(phone_numbers[2])
            if len(phone_numbers) >= 4:
                if phone_numbers[3] != None:
                    pager_phone.set_text(phone_numbers[3])

        email_address = self.object.get_email_addresses()
        if email_address:
            email1 = self.helper["email1"]
            if len(email_address) > 0:
                if email_address[0] != None:
                    email1.set_text(email_address[0])                           

        if self.object.get_birthdate():
            birthdate = self.helper["birthdate"]
            birthdate.set_text(self.object.get_birthdate())

        if self.object.get_image():
            imagefile = self.helper["imagefile"]
            imagefile.set_text(self.object.get_image().get_filename())

    def more_contact_info_tab(self,name):
        if name == "name":
            self.helper["names_cancel_button"].set_data("view",self)
            self.helper["names_save_button"].set_data("view",self)
            self.helper["notebook1"].set_page(2)
            
        elif name == "address":
            self.helper["address_cancel_button"].set_data("view",self)
            self.helper["address_save_button"].set_data("view",self)
            self.helper["notebook1"].set_page(3)
                
        elif name == "phone":
            self.helper["phones_cancel_button"].set_data("view",self)
            self.helper["phone_save_button"].set_data("view",self)
            self.helper["notebook1"].set_page(4)
            
        elif name == "email":
            self.helper["email_cancel_button"].set_data("view",self)
            self.helper["email_save_button"].set_data("view",self)
            self.helper["notebook1"].set_page(5)
            
        else:
            pass
        
    def save_contact(self):
        if GtkExtra.message_box(message='OK to save?',
                buttons=('OK', 'Cancel')) != 'OK':
                return

        firstname = self.helper["firstname"]
        lastname = self.helper["lastname"]
        middlename = self.helper["middlename"]
        titlename = self.helper["titlename"]
        companyname = self.helper["companyname"]
        nickname = self.helper["nickname"]

        name = [firstname.get_text(),lastname.get_text()]
        if middlename.get_text():
            name.append(middlename.get_text())
        else:
            name.append("")
        if titlename.get_text():
            name.append(titlename.get_text())
        else:
            name.append("")
        if companyname.get_text():
            name.append(companyname.get_text())
        else:
            name.append("")
        if nickname.get_text():
            name.append(nickname.get_text())
        else:
            name.append("")
            
        self.object.set_name(name)

        widget_array = [ self.helper["address1_1"],
                         self.helper["address2_1"],
                         self.helper["address3_1"],
                         self.helper["address4_1"] ]

        addresses = []
        i = 0
        
        for item in range(0,len(widget_array)):
            if widget_array[i].get_data("address"):
                address = widget_array[i].get_data("address")
                addresses.append(address)
            else:
                addresses.append(None)
            i+=1
                    
        self.object.set_physical_addresses(addresses)
        
        phone1 = self.helper["phone1"]
        work_phone = self.helper["work_phone"]
        cell_phone = self.helper["cell_phone"]
        pager_phone = self.helper["pager_phone"]

        phones = []
        if phone1.get_text():
            phones.append(phone1.get_text())
        else:
            phones.append(None)

        if work_phone.get_text():
            phones.append(work_phone.get_text())
        else:
            phones.append(None)

        if cell_phone.get_text():
            phones.append(cell_phone.get_text())
        else:
            phones.append(None)

        if pager_phone.get_text():
            phones.append(pager_phone.get_text())
        else:
            phones.append(None)

        self.object.set_phone_numbers(phones)
            
        email1 = self.helper["email1"]
        work_email = self.helper["work_email"]
        school_email = self.helper["school_email"]
        other_email = self.helper["other_email"]

        emails = []
        if email1.get_text():
            emails.append(email1.get_text())
        else:
            emails.append(None)

        if work_email.get_text():
            emails.append(work_email.get_text())
        else:
            emails.append(None)

        if school_email.get_text():
            emails.append(school_email.get_text())
        else:
            emails.append(None)

        if other_email.get_text():
            emails.append(other_email.get_text())
        else:
            emails.append(None)

        self.object.set_email_addresses(emails)
        
        birthdate = self.helper["birthdate"]
        if birthdate.get_text():
            self.object.set_birthdate(birthdate.get_text())
        else:
            self.object.set_birthdate(None)
           
        imagefile = self.helper["imagefile"]
        image = self.object.get_image()

        if imagefile.get_text():
            image.set_filename(imagefile.get_text())
        else:
            image.set_filename("images/nobody.jpeg")

        self.object.set_image(image)

        self.refresh_view()
        self.back_to_contact_view()
        
        self.notify_registered_callbacks("update")
        
    def back_to_contact_view(self):
        notebook = self.helper["notebook1"]
        notebook.set_page(0)

    def back_to_contact_edit(self,name):
        self.helper["notebook1"].set_page(1)

    def close_contact_window(self):
        window = self.helper["contactwindow"]
        window.destroy()

    def clear_directions(self,address_from,address_to):

        directions_list = []
        fastest_directions_list = self.helper["fastest_directions_list"]
        directions_list.append(fastest_directions_list)

        shortest_directions_list = self.helper["shortest_directions_list"]
        directions_list.append(shortest_directions_list)

        avoidhw_directions_list = self.helper["avoidhw_directions_list"]
        directions_list.append(avoidhw_directions_list)
        
        for directions_list_item in directions_list:
            directions_list_item.set_point(0)
            chars = directions_list_item.get_length()
            directions_list_item.forward_delete(chars)

        try:
            del address_from.stored_directions[address_to]
        except:
            pass
        try:
            del address_to.stored_directions[address_from]
        except:
            pass
        

    def show_directions(self,address_from,address_to):

        directions_back = self.helper["directions_back"]
        directions_back.set_data("view",self)
        directions_reverse = self.helper["directions_reverse"]
        directions_reverse.set_data("view",self)

        directions_clear = self.helper["directions_clear"]
        directions_clear.set_data("view",self)
        directions_clear.set_data("address_from",address_from)
        directions_clear.set_data("address_to",address_to)
        

        # Gather the directions
        try:
            directions = address_from.stored_directions[address_to]
        except:
            directions = address_from.get_directions(address_to)

        #
        # Seperate out the directions
        #
        directions_list = []
        
        fastest_directions_list = self.helper["fastest_directions_list"]
        fastest_directions = directions[0]
        directions_list.append([fastest_directions_list,fastest_directions])

        shortest_directions_list = self.helper["shortest_directions_list"]
        shortest_directions = directions[1]
        directions_list.append([shortest_directions_list,shortest_directions])

        avoidhw_directions_list = self.helper["avoidhw_directions_list"]
        avoidhw_directions = directions[2]
        directions_list.append([avoidhw_directions_list,avoidhw_directions])

        try:
            length = len(directions)
        except:
            length = 0

        if length > 1:
            address_from.stored_directions[address_to] = directions
        else:
            fastest_directions_list.insert_defaults("[No Directions Available]")
            shortest_directions_list.insert_defaults("[No Directions Available]")
            avoidhw_directions_list.insert_defaults("[No Directions Available]")
            return

        for directions_list_item,directions_item in directions_list:
            directions_list_item.set_point(0)
            chars = directions_list_item.get_length()
            directions_list_item.forward_delete(chars)
            directions_list_item.set_line_wrap(1)
            directions_list_item.set_word_wrap(1)
            directions_list_item.freeze()

            count = 0
            for item in directions_item:
                print count % 2
                if math.fmod(count,2):
                    directions_list_item.insert_defaults(item)
                    directions_list_item.insert_defaults("\n")
                else:
                    directions_list_item.insert_defaults(item)

                count+=1
            directions_list_item.thaw()
    
    def select_images(self):
        image = GtkExtra.file_sel_box()
        print image

    def notify_callback(self,type=None,object=None):

        if type == "address":
            widget = object.get_widget()
            #
            # Clear out the widget assignment
            #
            object.set_widget(None)
            
            widget.set_data("address",object)

            widget.set_point(0)
            chars = widget.get_length()
            widget.forward_delete(chars)
            widget.insert_defaults(object.get_address())

        elif type == "cancel":
            object.set_widget(None)
        else:
            pass
             
        
class GladeHelper:
    def __init__(self, glade_file, handler):
        self.widgets = libglade.GladeXML(glade_file)
        self.widgets.signal_autoconnect(handler.__dict__)

    def __getitem__(self, key):
        return self.widgets.get_widget(key)

class ContactSignalHandler:
    def on_edit_button_clicked(widget,*args):
        view = widget.get_data("view")
        view.edit_contact_tab()

    def on_cancel_button_clicked(widget,*args):
        view = widget.get_data("view")
        view.back_to_contact_view()

    def on_save_button_clicked(widget,*args):
        view = widget.get_data("view")
        view.save_contact()

    def on_close_button_clicked(widget,*args):
        view = widget.get_data("view")
        view.close_contact_window()

    def on_name_button_clicked(widget,*args):
        name = widget.get_data("name")
        view = widget.get_data("view")
        view.more_contact_info_tab(name)

    def on_address_button_clicked(widget,*args):
        name = widget.get_data("name")
        view = widget.get_data("view")
        view.more_contact_info_tab(name)

    def on_phone_button_clicked(widget,*args):
        name = widget.get_data("name")
        view = widget.get_data("view")
        view.more_contact_info_tab(name)

    def on_email_button_clicked(widget,*args):
        name = widget.get_data("name")
        view = widget.get_data("view")
        view.more_contact_info_tab(name)

    def on_more_cancel_button_clicked(widget,*args):
        view = widget.get_data("view")
        name = widget.get_data("name")
        view.back_to_contact_edit(name)

    def on_more_save_button_clicked(widget,*args):
        view = widget.get_data("view")
        name = widget.get_data("name")
        view.back_to_contact_edit(name)

    def on_address_combo_select(widget,*args):
        print "Item selected\n"
        
    def on_address_combo_changed(widget,*args):
        view = widget.get_data("view")
#        widget.emit_stop_by_name("changed")
        view.search_for_possible_addresses(widget)

    def on_address_list_select_child(widget,row):
        view = row.get_data("view")
        if view.object.get_owner():
            notebook = view.helper["notebook1"]
            notebook.set_page(6)
            address_to = row.get_data("address")
            address_from = view.object.get_owner().get_physical_addresses()[0]
            view.show_directions(address_from,address_to)
        else:
            if GtkExtra.message_box(message='Owner not set',
               buttons=('OK')) != 'OK':
               return


    def on_directions_back_clicked(widget,*args):
        view = widget.get_data("view")
        view.back_to_contact_view()

    def on_directions_reverse_clicked(widget,*args):
        view = widget.get_data("view")

        row = view.helper["address_list"].get_selection()[0]
        address_from = row.get_data("address")
        address_to = view.object.get_owner().get_physical_addresses()[0]
        view.show_directions(address_from,address_to)
        
    def on_imagebutton_clicked(widget,*args):
        view = widget.get_data("view")
        view.select_images()

    def on_directions_clear_clicked(widget,*args):
        view = widget.get_data("view")
        address_to = widget.get_data("address_to")
        address_from = widget.get_data("address_from")
        view.clear_directions(address_from,address_to)

    def on_address_activate(widget,*args):
        
        address_object = widget.get_data("address")
        contact_view = widget.get_data("view")
        
        if address_object:
            pass
        else:
            #
            # Create a new address object
            #
            address_object = address.Address()

        address_object.set_widget(widget)
        address_object.set_manager(contact_view.object.get_manager())

        contact_view.object.manager.register_object(address_object)
        new_view = address_object.new_view()
        new_view.view()
        new_view.register_callback(contact_view)

