from familiar.core import *
import gtk
import GtkExtra
import GDK
import libglade
import re
import string


class ContactManager(FamiliarClass):
    def __init__(self):
        self.contacts = []
        self.owner = None
        self.manager = None
        self.mimetype = "application/x-contactmanager"
        FamiliarClass.__init__(self)

    def set_manager(self, manager):
        self.manager = manager
        self.refresh_contacts()

    def get_manager(self):
        return self.manager

    def get_contacts(self):        
        self.refresh_contacts()
        return self.contacts[:]

    def get_owner(self):
        return self.owner
    
    def set_owner(self,owner):
        self.owner = owner
        
    def refresh_contacts(self):
        self.contacts = self.manager.search_by_type("application/x-contact")
        for contact in self.contacts:
            if len(contact.get_name()) < 1:
                self.manager.unregister_object(contact)
        self.contacts = self.manager.search_by_type("application/x-contact")
        self.sort_contacts()

    def new_view(self):
        return ContactManagerView(self)

    def search(self, search_text):
        matches = []
        contacts = self.get_contacts()
        expression = re.compile(search_text, re.IGNORECASE)
        
        for contact in contacts:
            name = contact.get_name()
            for component in name:
                m = expression.match(component)
                if m:
                    matches.append(contact)
                    break

        return matches

    def sort_contacts(self):
        for j in range(len(self.contacts) - 1):
            for i in range((j + 1), len(self.contacts)):
                if self.contacts[j].get_printable_name() > self.contacts[i].get_printable_name():
                    temp = self.contacts[j]
                    self.contacts[j] = self.contacts[i]
                    self.contacts[i] = temp
            

class ContactManagerView(FamiliarClassView):
    def __init__(self, object):
        self.object = object
        self.previous_search_string = ""
                
        FamiliarClassView.__init__(self)

    def view(self):
        self.helper = GladeHelper("familiar/contactmanager.glade",
                                  ContactManagerSignalHandler)

        contact_list = self.helper["contact_list"]
        
        contactmanager = self.helper["contactmanager"]
        contactmanager.set_default_size(240,320)
        
        width=gtk.screen_width()
        height=gtk.screen_height()
        if width >=640 and height >= 480:
            pass
        else:
            width=width-20
            height=height-40
            contactmanager.set_usize(width,height)
        

        search_entry = self.helper["search_entry"]
        new_menuitem = self.helper["new_menuitem"]
        delete_menuitem = self.helper["delete_menuitem"]
        set_owner_menuitem = self.helper["set_owner_menuitem"]

        contact_list.set_data("view", self)
        search_entry.set_data("view", self)
        new_menuitem.set_data("view",self)
        delete_menuitem.set_data("view",self)
        set_owner_menuitem.set_data("view",self)
        

        self.contacts_shown = self.object.get_contacts()
        self.refresh_view()

    def search_for(self, widget):
        contact_list = self.helper["contact_list"]
        search_string = widget.get_text()

        if search_string == self.previous_search_string:
            return

        self.previous_search_string = search_string
        matching_contacts = self.object.search(search_string)

        
        self.contacts_shown = matching_contacts
        self.refresh_view()
        
    def refresh_view(self):
        contact_list = self.helper["contact_list"]
        contact_list.clear()

        if not len(self.contacts_shown):
            contact_list.append(["[No Contacts Match Query]"])
            return

        owner = ""
        try:
            owner = self.object.get_owner()
        except:
            pass
        
        for match in self.contacts_shown:

            if owner == match:
                display = "%s*" % (match.get_printable_name())
            else:
                display = match.get_printable_name()
                
            row = contact_list.append([display])
            contact_list.set_row_data(row,match)
            
    def refresh_contact_list(self):
        self.object.refresh_contacts()
        self.contacts_shown = self.object.contacts
        self.refresh_view()
        
    def new_contact(self):
        
        contact = self.object.manager.new_object("application/x-contact")
        image = self.object.manager.new_object("image/jpeg")
        image.set_filename("images/nobody.jpeg")
        
        contact.set_image(image)
        contact.set_manager(self.object.manager)
        
        self.object.manager.register_object(contact)
        new_view = contact.new_view()
        new_view.view("edit")
        new_view.register_callback(self)
        
    def delete_contact(self):
        if GtkExtra.message_box(message='OK to delete?',
                buttons=('OK', 'Cancel')) != 'OK':
                return
            
        contact_selected = self.helper["contact_list"].selection
        row = contact_selected[0]
        contact = self.helper["contact_list"].get_row_data(row)
        self.object.manager.unregister_object(contact)
        self.refresh_contact_list()

    def set_mingle_owner(self):
        contact_selected = self.helper["contact_list"].selection
        if len(contact_selected):
            row = contact_selected[0]
            row_data = self.helper["contact_list"].get_row_data(row)
            if row_data == self.object.get_owner():
                # Owner is already set
                return

            self.object.owner = row_data
            self.refresh_contact_list()

        else:
            return
        
    def notify_callback(self, *args):
        self.refresh_contact_list()
        
class GladeHelper:
    def __init__(self, glade_file, handler):
        self.widgets = libglade.GladeXML(glade_file)
        self.widgets.signal_autoconnect(handler.__dict__)
    def __getitem__(self, key):
        return self.widgets.get_widget(key)

class ContactManagerSignalHandler:
    def on_contactmanager_window_destroy(*args):
        # XXX - Hrm.. shouldn't be here.
        gtk.mainquit()
        
    def on_contact_list_select_row(list, row, column, event):
        if event.type == GDK._2BUTTON_PRESS:
            contact = list.get_row_data(row)
            contact_manager = list.get_data("view")
            new_view = contact.new_view()
            new_view.view("view")
            new_view.register_callback(contact_manager)

    def on_search_entry_changed(widget, *args):
        view = widget.get_data("view")
        view.search_for(widget)

    def on_refresh_button_clicked(widget, *args):
        view = widget.get_data("view")
        view.refresh_contact_list()

    def on_new_menuitem_activate(widget,*args):
        view = widget.get_data("view")
        view.new_contact()

    def on_cancel_button_clicked(widget,*args):
        view = widget.get_data("view")
        view.back_to_contact_list()

    def on_save_button_clicked(widget,*args):
        view = widget.get_data("view")
        view.save_contact()

    def on_delete_menuitem_activate(widget,*args):
        view = widget.get_data("view")
        view.delete_contact()

    def on_set_owner_menuitem_activate(widget,*args):
        view = widget.get_data("view")
        view.set_mingle_owner()
