from familiar.core import *
import gtk
import GDK
import libglade
import re
import string
import time

not_available_string = "[n/a]"

class EventManager(FamiliarClass):
    def __init__(self):
        self.events = []
        self.mimetype = "application/x-eventmanager"
        FamiliarClass.__init__(self)
        

    def set_manager(self, manager):
        self.manager = manager

    def get_events(self):        
        return self.events[:]

    def get_events_by_startday(self,day):
###        print "events for startday ",day," requested"
        events = []
        for event in self.events:
            eventstart = event.get_start_date()
            try:
                if eventstart[0] == day[0]:
                    if eventstart[1] == day[1]:
                        if eventstart[2] == day[2]:
                            events.append(event)
            except:
                print 'comparison failed'
###        print "found ",len(events)," events"
        return events

    def get_events_by_starthour(self,yeardayhour):
###        print "events for starthour ",yeardayhour," requested"
        fevents=[]
        events=self.get_events_by_startday(yeardayhour[0:3])
        for event in events:
            eventstart = event.get_start_date()
            if eventstart[3] == yeardayhour[3]:
                fevents.append(event)
###        print "found ",len(fevents)," fevents"
        return fevents
        

    def new_view(self):
        return EventManagerView(self)

    def refresh_events(self):
        self.events = self.manager.search_by_type("application/x-event")


class EventManagerView(FamiliarClassView):
    def __init__(self, object):
        self.object = object
        self.month_changed = 0
        FamiliarClassView.__init__(self)

    def view(self):
        self.helper = GladeHelper("familiar/eventmanager.glade",
                                  EventManagerSignalHandler)

        save_event_button = self.helper["save_event_button"]
        month_display = self.helper["month_display"]
        day_clist = self.helper["day_clist"]
        save_event_button.set_data("view",self)
        month_display.set_data("view",self)
        day_clist.set_data("view",self)

        self.display_tab = self.helper["display_tab"]
        self.helper["discard_event_button"].set_data("view",self)
        self.helper["next_day_button"].set_data("view",self)
        self.helper["prev_day_button"].set_data("view",self)
        self.object.refresh_events()
        self.set_day_shown(None)
        self.switch_display_day()

    def switch_display_day(self):
        self.display_tab.set_page(1)

    def set_day_shown(self,day):
        if day == None:
            day=time.localtime(time.time())
        evtime=EventTime(day)
        self.helper["next_day_button"].set_data("day",evtime)
        self.helper["prev_day_button"].set_data("day",evtime)
        events=self.object.get_events_by_startday(day)
        day_clist = self.helper["day_clist"]
        day_clist.clear()
        hour=5
        rowno=0
        for hour in range(0,24):
            matching_ev=0
            for ev in events:
                t = ev.get_start_date()
                if t[3] == hour:
                    day_clist.append(["%02d:00" % hour, "%s" % (ev.get_summary())])
                    matching_ev=1
                    rowdata=(day[0],day[1],day[2],hour,0)
                    day_clist.set_row_data(rowno,rowdata)
                    rowno+=1
            if matching_ev == 0:
                day_clist.append(["%02d:00" % hour, ""])
                rowdata=(day[0],day[1],day[2],hour,0)
                day_clist.set_row_data(rowno,rowdata)
                rowno+=1

        day_label = self.helper["day_displayed"]
        day_label.set_text("%d-%02d-%02d" % (day[0], day[1], day[2])) 
        day_label.set_data("day",day)

    def switch_display_event(self):
        self.display_tab.set_page(2)

    def set_event_shown(self,event):
        editable=self.helper["event_display_text"]
        self.helper["discard_event_button"].set_data("event",event)
        self.helper["save_event_button"].set_data("event",event)

        editable.set_point(0)
        chars = editable.get_length()
        editable.forward_delete(chars)

        if event.get_description():
            editable.insert_defaults(event.get_description())
            
        evtime=EventTime(event.get_start_date())
        try:
            evetime=EventTime(event.get_end_date())
        except:
            evetime=EventTime(event.get_start_date())
        self.helper["starts-entry"].set_text(evtime.get_printable_time())
        self.helper["ends-combo"].set_popdown_strings(evetime.adjacent_str())

    def new_event(self):
        event = self.object.manager.new_object("application/x-event")
        self.object.manager.register_object(event)
        return event

    def discard_event(self,event):
        self.object.manager.unregister_object(event)
        self.object.refresh_events()


class GladeHelper:
    def __init__(self, glade_file, handler):
        self.widgets = libglade.GladeXML(glade_file)
        self.widgets.signal_autoconnect(handler.__dict__)
        
    def __getitem__(self, key):
        return self.widgets.get_widget(key)

class EventManagerSignalHandler:
    def on_window_delete(*args):
        # XXX - Hrm.. shouldn't be here.
        gtk.mainquit()

    def on_month_display_day_selected(widget, *args):
        view = widget.get_data("view")
        wday=widget.get_date()
        # convert month here, to avoid some confusion later...
        day=[wday[0],wday[1]+1,wday[2]]
        view.set_day_shown(day)
        if view.month_changed == 0:
            view.switch_display_day()
        else:
            view.month_changed=0

    def on_month_display_month_changed(widget, *args):
        view = widget.get_data("view")
        view.month_changed=1

    def on_day_clist_select_row(widget,row,col,*args):
        view=widget.get_data("view")
        widget.unselect_all()
        daytime=widget.get_row_data(row)
        events = []
        events = view.object.get_events_by_starthour(daytime)
        wtext_summary=widget.get_text(row,col)
        if wtext_summary == "":
            wtext_summary= not_available_string
        event=None
        if  col > 0 and events != []:  
            for ev in events:
                ####  print wtext_summary,"==",ev.get_summary(),"?"
                if ev.get_summary() == wtext_summary:
                    event=ev
            if event==None:
                print "no matching event?!?"
        else:
            event = view.new_event()
            event.set_start_date(daytime)
        view.set_event_shown(event)
        view.switch_display_event()
        

    def on_discard_event_button_clicked(widget,*args):
        event = widget.get_data("event")
        try:
            id = event.get_id()
        except:
            print "No event here..."
            return
        view = widget.get_data("view")
        day=event.get_start_date()
        view.discard_event(event)
        view.set_day_shown(day)
        view.switch_display_day()
        widget.set_data("event",None)

    def on_save_event_button_clicked(widget,*args):
        view= widget.get_data("view")
        event=widget.get_data("event")
        editable=view.helper["event_display_text"]
        startentry=view.helper["starts-entry"]
        endcombo=view.helper["ends-combo"]
        evtext=editable.get_chars(0,-1)
        startdate=EventTime(startentry.get_text())
        endsdate=EventTime(endcombo.entry.get_text())
        event.set_start_date(startdate.get_time())
        event.set_end_date(endsdate.get_time())
        event.set_description(evtext)
        view.object.refresh_events()
        evstart=event.get_start_date()
        view.set_day_shown(evstart[0:3])
        view.switch_display_day()
        events = view.object.get_events()
        
    def on_prev_day_button_clicked(widget, *args):
        view=widget.get_data("view")
        day=widget.get_data("day")
        view.set_day_shown(day.prev_day().get_time())

    def on_next_day_button_clicked(widget, *args):
        view=widget.get_data("view")
        day=widget.get_data("day")
        view.set_day_shown(day.next_day().get_time())

# Helper Classes

class EventTime:
    def __init__(self,timein):
        self.timedate=[]
        try:
            if len(timein) < 10:
                isstr=0
            else:
                isstr=1
        except:
            isstr=1

        if isstr == 0:
            for n in range(0,5):
                try:
                    self.timedate.append(timein[n])
                except:
                    self.timedate.append(0)
        else:
            try:
                tmp=string.split(timein,' ',1)
                minute=string.split(tmp[1],':',1)
                day=string.split(tmp[0],'-',2)
            except AttributeError:
                raise GenericError("bad timein")
            try:
                self.timedate.append(int(day[0]))
                self.timedate.append(int(day[1]))
                self.timedate.append(int(day[2]))
            except:
                print 'invalid time string format!'

            try:
                self.timedate.append(int(minute[0]))
                self.timedate.append(int(minute[1]))
            except:
                print 'invalid string time format!'
                
        self.strformat="%d-%02d-%02d %02d:%02d"
        
    def get_time(self):
        return self.timedate

    def get_printable_time(self):
        tmptime = self.get_time()
        return self.strformat % (tmptime[0],tmptime[1],tmptime[2],tmptime[3],tmptime[4])

    def adjacent_str(self):
        ret=[]
        tmptime=[0,0,0,0,0]
        secs=time.mktime((self.timedate[0],self.timedate[1],self.timedate[2],self.timedate[3],self.timedate[4],0,0,0,-1))

        for i in range(1,48):
            tmptime=time.localtime(secs+(i*30*60))
            if tmptime[3]  >= 23:
                break
            ret.append(self.strformat % ( tmptime[0],tmptime[1],tmptime[2],tmptime[3],tmptime[4]))
        return ret

    def prev_hour(self):
        if self.timedate[3] > 0:
            self.timedate[3]-=1
        else:
            self.timedate[2]-=1
            self.timedate[3]+=23
        return self

    def next_hour(self):
        if self.timedate[3] < 12:
            self.timedate[3]+=1
        else:
            self.timedate[2]+=1
            self.timedate[3]-=23
        return self

    def prev_day(self):
        secs=time.mktime((self.timedate[0],self.timedate[1],self.timedate[2],self.timedate[3],self.timedate[4],0,0,0,-1))
        self.timedate=time.localtime(secs-(3600*24))
        return self

    def next_day(self):
        secs=time.mktime((self.timedate[0],self.timedate[1],self.timedate[2],self.timedate[3],self.timedate[4],0,0,0,-1))
        self.timedate=time.localtime(secs+(3600*24))
        return self

class GenericError:
    def __init__(self, value):
        self.value = value
        
    def __str__(self):
        return self.value
