import pickle

from familiar.core import *

# Import all types we want to support.. this will change when we have
# per class registration support.

import familiar.image
import familiar.contact
import familiar.contactmanager
import familiar.task
import familiar.taskmanager
import familiar.event
import familiar.eventmanager
import familiar.schedule
import familiar.schedulemanager
import familiar.address

class ObjectManager(FamiliarClass):
    def __init__(self):
        self.type_list = []
        self.type_to_object_list = {}
        
        FamiliarClass.__init__(self)


    def register_object(self, object):

        object_type = object.get_mimetype()

        try:
            object_list = self.type_to_object_list[object_type]
        except:
            object_list = []

        object_list.append(object)

        self.type_to_object_list[object_type] = object_list

    def unregister_object(self, object):
# Really hoaky, but it should work for now
        object_type = object.get_mimetype()

        try:
            object_list = self.type_to_object_list[object_type]
        except:
            object_list = []
        
        for i in range(0,len(object_list)):
            if object_list[i] == object:
                del object_list[i]
                break


    def add_type(self, new_type):
        for type in self.type_list:
            if type == new_type:
                return 1

        self.type_list.append(new_type)
        return 1
    

    def search_by_type(self, object_type):
        
        try:
            object_list = self.type_to_object_list[object_type]
        except:
            object_list = []
            
        return object_list[:]

    def new_object(self, mimetype):
        if mimetype == "image/jpeg":
            return familiar.image.JPEG()

        if mimetype == "application/x-contact":
            return familiar.contact.Contact()

        if mimetype == "application/x-task":
            return familiar.task.Task()

        if mimetype == "application/x-event":
            return familiar.event.Event()

        if mimetype == "application/x-scheduleitem":
            return familiar.scheduleitem.ScheduleItem()

        if mimetype == "application/x-contactmanager":
            return familiar.contactmanager.ContactManager()

        if mimetype == "application/x-taskmanager":
            return familiar.taskmanager.TaskManager()

        if mimetype == "application/x-eventmanager":
            return familiar.eventmanager.EventManager()

        if mimetype == "application/x-schedulemanager":
            return familiar.schedulemanager.ScheduleManager()

        if mimetype == "application/x-streetaddress":
            return familiar.address.Address()
        
        return None
    
    def dump(self, filename):
        file = open(filename, "w+")
        
        pickle.dump(self, file)
        file.close()

def load_manager(filename):
        file = open(filename, "r")
        return pickle.load(file)
    
