from familiar.core import *
import gtk
import GDK
import libglade
import re
import string
import time


class ScheduleManager(FamiliarClass):
    def __init__(self):
        self.schedules = []
        self.mimetype = "application/x-schedulemanager"
        FamiliarClass.__init__(self)
        
    def set_manager(self, manager):
        self.manager = manager
        self.refresh_schedules()

    def get_schedules(self):        
        return self.schedules[:]

    def refresh_schedules(self):
        self.schedules = self.manager.search_by_type("application/x-scheduleitem")

    def new_view(self):
        return ScheduleManagerView(self)

class ScheduleManagerView(FamiliarClassView):
    def __init__(self, object):
        self.object = object
        self.previous_search_string = ""
                
        FamiliarClassView.__init__(self)

    def view(self):
        self.helper = GladeHelper("familiar/schedulemanager.glade",
                                  ScheduleManagerSignalHandler)

        calendar = self.helper["calendar"]
        calendar.set_data("view",self)
        current_time = time.time()
        (year,month,month_day,hour,minute,second,weekday,year_day,dst) = time.localtime(current_time)
        calendar.select_month(month-1,year)
        calendar.select_day(month_day)

            
    def refresh_schedule_list(self):
        self.object.refresh_contacts()
        self.contacts_shown = self.object.contacts
        self.refresh_view()
        
    def new_scheduleitem(self):
        calendar = self.helper["calendar"]
        print "New schedule item\n"
        print calendar.get_date()
        
    def update_items_for_day(self):
        calendar = self.helper["calendar"]
        print calendar.get_date()
        
    def notify_callback(self, *args):
        self.refresh_contact_list()
        
class GladeHelper:
    def __init__(self, glade_file, handler):
        self.widgets = libglade.GladeXML(glade_file)
        self.widgets.signal_autoconnect(handler.__dict__)
        
    def __getitem__(self, key):
        return self.widgets.get_widget(key)

class ScheduleManagerSignalHandler:
    def on_schedulemanager_window_destroy(*args):
        # XXX - Hrm.. shouldn't be here.
        gtk.mainquit()
        
    def on_contact_list_select_row(list, row, column, event):
        if event.type == GDK._2BUTTON_PRESS:
            contact = list.get_row_data(row)
            contact_manager = list.get_data("view")
            new_view = contact.new_view()
            new_view.view("view")
            new_view.register_callback(contact_manager)

    def on_calendar_day_selected(widget, *args):
        view = widget.get_data("view")
        view.update_items_for_day()
        
    def on_calendar_day_double_clicked(widget, *args):
        view = widget.get_data("view")
        view.new_scheduleitem()

    def on_search_entry_changed(widget, *args):
        view = widget.get_data("view")
        view.search_for(widget)

    def on_refresh_button_clicked(widget, *args):
        view = widget.get_data("view")
        view.refresh_contact_list()

    def on_new_button_clicked(widget,*args):
        view = widget.get_data("view")
        view.new_contact()

    def on_cancel_button_clicked(widget,*args):
        view = widget.get_data("view")
        view.back_to_contact_list()

    def on_save_button_clicked(widget,*args):
        view = widget.get_data("view")
        view.save_contact()

    def on_delete_button_clicked(widget,*args):
        view = widget.get_data("view")
        view.delete_contact()
