import gtk
import libglade

from familiar.core import *

class Task(FamiliarClass):
    def __init__(self):
        self.mimetype = "application/x-task"
        self.priority = 0
        self.label = ""
        self.description = ""
        FamiliarClass.__init__(self)

    def set_priority(self, priority):
        self.priority = priority

    def get_priority(self):
        return self.priority

    def set_label(self, label):
        self.label = label

    def get_label(self):
        return self.label

    def set_description(self, description):
        self.description = description

    def get_description(self):
        return self.description

    def new_view(self):
        return TaskView(self)


class TaskView(FamiliarClassView):
    def __init__(self, object):
        self.object = object
        self.callback_object_list = []
        FamiliarClassView.__init__(self)

    def register_callback(self, callback_object, type=None):
        self.callback_object_list.append(callback_object)

    def notify_registered_callbacks(self, *args):
        for callback in self.callback_object_list:
            callback.notify_callback(self)
        
    def view(self):
        self.helper = GladeHelper("familiar/task.glade",
                                  TaskSignalHandler)
        
        cancel_button = self.helper["cancel_button"]
        cancel_button.set_data("view",self)

        save_button = self.helper["save_button"]
        save_button.set_data("view",self)
        
        self.refresh_view()
        
    def refresh_view(self):
        label_entry = self.helper["label_entry"]
        priority_entry = self.helper["priority_entry"]
        description_text = self.helper["description_text"]

        label_entry.set_text(self.object.get_label())

        priority_entry.set_text("%d" % (self.object.get_priority()))
        description_text.freeze()
        description_text.set_point(0)
        description_text.forward_delete(description_text.get_length())
        description_text.insert_defaults(self.object.get_description())
        description_text.thaw()

    def close_window(self):
        window = self.helper["taskwindow"]
        window.destroy()

    def save_task(self):
        label_entry = self.helper["label_entry"]
        priority_entry = self.helper["priority_entry"]
        description_text = self.helper["description_text"]
        
        self.object.set_priority (int(priority_entry.get_text()))
        self.object.set_label (label_entry.get_text())
        self.object.set_description (description_text.get_chars(0,description_text.get_length()))
        
        self.close_window()
        self.notify_registered_callbacks()

class GladeHelper:
    def __init__(self, glade_file, handler):
        self.widgets = libglade.GladeXML(glade_file)
        self.widgets.signal_autoconnect(handler.__dict__)

    def __getitem__(self, key):
        return self.widgets.get_widget(key)

class TaskSignalHandler:
    def on_cancel_button_clicked(widget,*args):
        view = widget.get_data("view")
        view.close_window()

    def on_save_button_clicked(widget,*args):
        view = widget.get_data("view")
        view.save_task()
