from familiar.core import *
import gtk
import GDK
import libglade


class TaskManager(FamiliarClass):
    def __init__(self):
        self.tasks = []
        self.mimetype = 'application/x-taskmanager'
        FamiliarClass.__init__(self)
    
    def set_manager(self, manager):
        self.manager = manager
        self.refresh_tasks()

    def get_tasks(self):
        return self.tasks

    def refresh_tasks(self):
        self.tasks = self.manager.search_by_type("application/x-task")
        for task in self.tasks:
            if len(task.get_label()) < 1:
                self.manager.unregister_object(task)
        self.sort_tasks()

    def sort_tasks(self):
        for j in range(len(self.tasks) - 1):
            for i in range((j + 1), len(self.tasks)):
                if self.tasks[j].get_priority() > self.tasks[i].get_priority():
                    temp = self.tasks[j]
                    self.tasks[j] = self.tasks[i]
                    self.tasks[i] = temp

    def new_view(self):
        return TaskManagerView(self)

class TaskManagerView(FamiliarClassView):
    def __init__(self, object):
        self.object = object
        FamiliarClassView.__init__(self)

    def view(self):
        self.helper = GladeHelper("familiar/taskmanager.glade",
                                  TaskManagerSignalHandler)

        new_button = self.helper["new_button"]
        refresh_button = self.helper["refresh_button"]
        delete_button = self.helper["delete_button"]
        task_clist = self.helper["task_clist"]
        
        new_button.set_data("view",self)
        refresh_button.set_data("view",self)
        delete_button.set_data("view",self)
        task_clist.set_data("view",self)
        
        self.refresh_view()
    
    def refresh_view(self):
        task_clist = self.helper["task_clist"]
        task_clist.clear()
        self.object.refresh_tasks()
        tasks = self.object.get_tasks()
        for task in tasks:
            row = task_clist.append(["%d" % (task.get_priority()),
                                     task.get_label()])
            task_clist.set_row_data(row, task)

    def refresh_task_list(self):
        self.object.refresh_tasks()
        self.refresh_view()
        
    def new_task(self):
        task = self.object.manager.new_object("application/x-task")
        self.object.manager.register_object(task)
        new_view = task.new_view()
        new_view.view()
        new_view.register_callback(self)

    def delete_task(self):
        task_selected = self.helper["task_clist"].selection
        if not task_selected:
            return
        row = task_selected[0]
        task = self.helper["task_clist"].get_row_data(row)
        self.object.manager.unregister_object(task)
        self.refresh_task_list()
    
    def notify_callback(self, *args):
        self.refresh_task_list()

class GladeHelper:
    def __init__(self, glade_file, handler):
        self.widgets = libglade.GladeXML(glade_file)
        self.widgets.signal_autoconnect(handler.__dict__)

        
    def __getitem__(self, key):
        return self.widgets.get_widget(key)

class TaskManagerSignalHandler:
    def on_taskmanager_window_destroy(*args):
        # XXX - Hrm.. shouldn't be here.
        gtk.mainquit()
        
    def on_task_clist_select_row(clist, row, column, event):
        if event.type == GDK._2BUTTON_PRESS:
            task = clist.get_row_data(row)
            task_manager = clist.get_data("view")
            new_view = task.new_view()
            new_view.view()
            new_view.register_callback (task_manager)

    def on_refresh_button_clicked(widget, *args):
        view = widget.get_data("view")
        view.refresh_task_list()

    def on_new_button_clicked(widget,*args):
        view = widget.get_data("view")
        view.new_task()

    def on_cancel_button_clicked(widget,*args):
        view = widget.get_data("view")
        view.back_to_task_list()

    def on_save_button_clicked(widget,*args):
        view = widget.get_data("view")
        view.save_task()

    def on_delete_button_clicked(widget,*args):
        view = widget.get_data("view")
        view.delete_task()

        



